@@ -33,7 +33,8 @@
  */
 
 use std::io::{Error, ErrorKind, Read, Seek, SeekFrom};
-use std::{fs, os::unix::prelude::FileExt, path::PathBuf};
+use std::{fs, thread};
+use std::{os::unix::prelude::FileExt, path::PathBuf, time::Duration};
 
 use std::io::prelude::*;
 
@@ -100,27 +101,46 @@ fn mount_part(part: &str, mountpath: &PathBuf) -> Result<Mount, Error> {
     if !srcpath.exists() {
         srcpath.set_file_name(&part_a);
     }
+    if !srcpath.exists() {
+        srcpath.set_file_name(&part_b);
+    }
     if !srcpath.exists() {
         srcpath = PathBuf::from(PARTLABEL_DIR).join(part);
     }
     if !srcpath.exists() {
         srcpath.set_file_name(&part_a);
     }
+    if !srcpath.exists() {
+        srcpath.set_file_name(&part_b);
+    }
+    if !srcpath.exists() {
+        let err_str = format!("Unable to find device file for partition {part}!");
+        error!("{err_str}");
+        return Err(Error::new(ErrorKind::NotFound, err_str));
+    }
 
-    debug!("Attempting to mount {}", srcpath.display());
+    debug!(
+        "Attempting to mount {} to {}",
+        srcpath.display(),
+        mountpath.display()
+    );
 
     match Mount::builder().flags(flags).mount(&srcpath, mountpath) {
         Ok(m) => Ok(m),
         Err(e) => {
+            error!(
+                "Unable to mount {} on {}: {}",
+                srcpath.display(),
+                mountpath.display(),
+                e
+            );
             if srcpath.ends_with(&part_a) {
-                error!(
-                    "Unable to mount {} on {}: {}",
-                    srcpath.display(),
-                    mountpath.display(),
-                    e
-                );
                 srcpath.set_file_name(&part_b);
-                info!("Mounting {} on {}", srcpath.display(), mountpath.display());
+                debug!(
+                    "Attempting to mount {} to {}",
+                    srcpath.display(),
+                    mountpath.display()
+                );
                 Mount::builder().flags(flags).mount(&srcpath, mountpath)
             } else {
                 Err(e)
@@ -208,7 +228,7 @@ fn map_dynpart(part: &str) -> Result<(), Error> {
     }
 }
 
-pub fn process(config: Config) -> Result<Status, Error> {
+pub fn process(config: Config, extract_path: &String) -> Result<Status, Error> {
     let mut files: Vec<String> = Vec::new();
     let mut folders: Option<Vec<String>> = None;
 
@@ -217,22 +237,40 @@ pub fn process(config: Config) -> Result<Status, Error> {
         info!("Mounting {part} as the 'super' partition");
         let mut success = false;
 
+        /*
+         * Systems using dynparts right from their initial release usually
+         * just have one super partition. On the other hand, systems "converted"
+         * to dynparts during their lifetime will likely have 2 super partitions,
+         * with the possibility that both of them are valid but only one holds
+         * actual filesystems.
+         * Mitigate failure risks by mapping all potential super partitions we
+         * can find.
+         */
         for suffix in ["", "_a", "_b"] {
             let testpart = format!("{part}{suffix}");
             debug!("Attempting to map dynpart {testpart}");
             if map_dynpart(&testpart).is_ok() {
                 success = true;
-                break;
             }
         }
 
         if !success {
             return Err(Error::other("Failed to map super partition!"));
+        } else {
+            // Wait up to 500ms to ensure mapped partitions appear under /dev/mapper
+            for _ in 0..5 {
+                if let Ok(mapped) = fs::read_dir("/dev/mapper") {
+                    if mapped.count() > 1 {
+                        break;
+                    }
+                }
+                thread::sleep(Duration::from_millis(100));
+            }
         }
     }
 
     for entry in config.firmware {
-        let destpath = PathBuf::from("/lib/firmware/updates").join(entry.destination);
+        let destpath = PathBuf::from(extract_path).join(entry.destination);
 
         if let Err(e) = fs::create_dir_all(&destpath) {
             warn!("Unable to create folder {}: {}", destpath.display(), e);
@@ -366,7 +404,7 @@ pub fn process(config: Config) -> Result<Status, Error> {
                 "Processing partition {} for raw dump",
                 entry.partition.as_str()
             );
-            let destpath = PathBuf::from("/lib/firmware/updates").join(&entry.destination);
+            let destpath = PathBuf::from(extract_path).join(&entry.destination);
             if let Err(e) = fs::create_dir_all(&destpath) {
                 warn!("Unable to create folder {}: {}", destpath.display(), e);
                 continue;
