@@ -11,6 +11,8 @@ use std::{fs, path::PathBuf};
 use clap::Parser;
 use serde::Deserialize;
 
+const DEFAULT_EXTRACT_PATH: &str = "/lib/firmware/updates";
+
 const STATUS_FILE_PATH: &str = "/var/lib/droid-juicer/status.json";
 const CONFIG_DIR_PATH: &str = "/usr/share/droid-juicer/configs";
 const CONFIG_FILE_PATH: &str = "/etc/droid-juicer/config.toml";
@@ -32,13 +34,40 @@ struct Config {
     juicer: firmware::Config,
 }
 
-#[derive(Deserialize, Default)]
+#[derive(Deserialize, PartialEq, Debug)]
+#[serde(default)]
+struct GeneralConfig {
+    extract_path: String,
+}
+
+impl Default for GeneralConfig {
+    fn default() -> Self {
+        let extract_path = match fs::read_to_string("/sys/module/firmware_class/parameters/path") {
+            Ok(firmware_class_path) => {
+                let path = firmware_class_path.trim_end();
+                if !path.is_empty() {
+                    path.to_string()
+                } else {
+                    DEFAULT_EXTRACT_PATH.to_string()
+                }
+            }
+            Err(_) => DEFAULT_EXTRACT_PATH.to_string(),
+        };
+
+        Self { extract_path }
+    }
+}
+
+#[derive(Deserialize, Default, PartialEq, Debug)]
+#[serde(default)]
 struct PostProcessConfig {
     commands: Vec<String>,
 }
 
-#[derive(Deserialize, Default)]
+#[derive(Deserialize, Default, PartialEq, Debug)]
+#[serde(default)]
 struct MainConfig {
+    general: GeneralConfig,
     postprocess: PostProcessConfig,
 }
 
@@ -71,7 +100,8 @@ fn detect_device() -> Result<String, Error> {
 
 fn main() -> Result<(), Error> {
     let opt = Opt::parse();
-    let mut main_config = MainConfig::default();
+
+    pretty_env_logger::init();
 
     let device = match opt.device {
         Some(str) => str,
@@ -86,11 +116,10 @@ fn main() -> Result<(), Error> {
         }
     };
 
-    if PathBuf::from(CONFIG_FILE_PATH).exists() {
-        if let Ok(contents) = fs::read_to_string(CONFIG_FILE_PATH) {
-            main_config = toml::from_str(contents.as_str())?;
-        }
-    }
+    let main_config = match fs::read_to_string(CONFIG_FILE_PATH) {
+        Ok(contents) => toml::from_str(contents.as_str()).unwrap(),
+        Err(_) => MainConfig::default(),
+    };
 
     if opt.cleanup {
         info!("Cleaning up files for device {device}");
@@ -127,7 +156,7 @@ fn main() -> Result<(), Error> {
 
         let config: Config = toml::from_str(contents.as_str()).unwrap();
         debug!("Extracting firmware for device {device}");
-        let status = firmware::process(config.juicer)?;
+        let status = firmware::process(config.juicer, &main_config.general.extract_path)?;
         debug!("Writing status file");
         fs::create_dir_all("/var/lib/droid-juicer/")?;
         if let Ok(f) = fs::File::create(STATUS_FILE_PATH) {
@@ -154,3 +183,54 @@ fn main() -> Result<(), Error> {
 
     Ok(())
 }
+
+#[cfg(test)]
+mod tests {
+    use super::*;
+
+    #[test]
+    fn default_main_config() {
+        let firmware_class_path =
+            fs::read_to_string("/sys/module/firmware_class/parameters/path").unwrap();
+
+        let expected_config = MainConfig {
+            general: GeneralConfig {
+                extract_path: if firmware_class_path.trim().is_empty() {
+                    DEFAULT_EXTRACT_PATH.to_string()
+                } else {
+                    firmware_class_path.trim().to_string()
+                },
+            },
+            postprocess: PostProcessConfig {
+                commands: Vec::new(),
+            },
+        };
+
+        assert_eq!(toml::from_str::<MainConfig>("").unwrap(), expected_config);
+    }
+
+    #[test]
+    fn custom_main_config() {
+        let config_text = r#"
+            [general]
+            extract_path = "/var/lib/firmware-extract"
+
+            [postprocess]
+            commands = [ "/usr/bin/true" ]
+        "#;
+
+        let expected_config = MainConfig {
+            general: GeneralConfig {
+                extract_path: "/var/lib/firmware-extract".to_string(),
+            },
+            postprocess: PostProcessConfig {
+                commands: vec!["/usr/bin/true".to_string()],
+            },
+        };
+
+        assert_eq!(
+            toml::from_str::<MainConfig>(config_text).unwrap(),
+            expected_config
+        );
+    }
+}
